@@ -11,11 +11,13 @@
 #include <vtkm/cont/UncertainArrayHandle.h>
 #include <vtkm/cont/UnknownArrayHandle.h>
 
+#include <vtkm/cont/ArrayCopy.h>
 #include <vtkm/cont/ArrayHandleCast.h>
 #include <vtkm/cont/ArrayHandleConstant.h>
 #include <vtkm/cont/ArrayHandleCounting.h>
 #include <vtkm/cont/ArrayHandleGroupVecVariable.h>
 #include <vtkm/cont/ArrayHandleMultiplexer.h>
+#include <vtkm/cont/ArrayHandleRuntimeVec.h>
 
 #include <vtkm/TypeTraits.h>
 
@@ -43,27 +45,6 @@ struct UnusualType
   operator T() const { return this->X; }
 };
 
-} // anonymous namespace
-
-namespace vtkm
-{
-
-// UnknownArrayHandle requires its value type to have a defined VecTraits
-// class. One of the tests is to use an "unusual" array.
-// Make an implementation here. Because I am lazy, this is only a partial
-// implementation.
-template <>
-struct VecTraits<UnusualType> : VecTraits<UnusualType::T>
-{
-  using ComponentType = UnusualType;
-  using BaseComponentType = UnusualType;
-};
-
-} // namespace vtkm
-
-namespace
-{
-
 const vtkm::Id ARRAY_SIZE = 10;
 
 struct CheckFunctor
@@ -90,8 +71,8 @@ struct CheckFunctor
   void operator()(const vtkm::cont::ArrayHandle<T, S>& array, bool& called) const
   {
     called = true;
-    std::cout << "  Checking for array type " << typeid(T).name() << " with storage "
-              << typeid(S).name() << std::endl;
+    std::cout << "  Checking for array type " << vtkm::cont::TypeToString<T>() << " with storage "
+              << vtkm::cont::TypeToString<S>() << std::endl;
 
     CheckArray(array);
   }
@@ -100,6 +81,8 @@ struct CheckFunctor
 void BasicUnknownArrayChecks(const vtkm::cont::UnknownArrayHandle& array,
                              vtkm::IdComponent numComponents)
 {
+  std::cout << "  Checking an UnknownArrayHandle containing " << array.GetArrayTypeName()
+            << std::endl;
   VTKM_TEST_ASSERT(array.GetNumberOfValues() == ARRAY_SIZE,
                    "Dynamic array reports unexpected size.");
   VTKM_TEST_ASSERT(array.GetNumberOfComponentsFlat() == numComponents,
@@ -369,6 +352,43 @@ void TryAsMultiplexer(vtkm::cont::UnknownArrayHandle sourceArray)
 #endif
 }
 
+struct SimpleRecombineCopy
+{
+  template <typename T>
+  void operator()(const vtkm::cont::ArrayHandleRecombineVec<T>& inputArray,
+                  const vtkm::cont::UnknownArrayHandle& output) const
+  {
+    vtkm::cont::ArrayHandleRecombineVec<T> outputArray =
+      output.ExtractArrayFromComponents<T>(vtkm::CopyFlag::Off);
+    vtkm::Id size = inputArray.GetNumberOfValues();
+    outputArray.Allocate(size);
+    auto inputPortal = inputArray.ReadPortal();
+    auto outputPortal = outputArray.WritePortal();
+
+    for (vtkm::Id index = 0; index < size; ++index)
+    {
+      outputPortal.Set(index, inputPortal.Get(index));
+    }
+  }
+};
+
+template <typename T>
+void TryExtractArray(const vtkm::cont::UnknownArrayHandle& originalArray)
+{
+  // This check should already have been performed by caller, but just in case.
+  CheckUnknownArray<vtkm::List<T>, VTKM_DEFAULT_STORAGE_LIST>(originalArray,
+                                                              vtkm::VecTraits<T>::NUM_COMPONENTS);
+
+  std::cout << "Create new instance of array." << std::endl;
+  vtkm::cont::UnknownArrayHandle newArray = originalArray.NewInstanceBasic();
+
+  std::cout << "Do CastAndCallWithExtractedArray." << std::endl;
+  originalArray.CastAndCallWithExtractedArray(SimpleRecombineCopy{}, newArray);
+
+  CheckUnknownArray<vtkm::List<T>, VTKM_DEFAULT_STORAGE_LIST>(newArray,
+                                                              vtkm::VecTraits<T>::NUM_COMPONENTS);
+}
+
 template <typename T>
 void TryDefaultType()
 {
@@ -379,6 +399,8 @@ void TryDefaultType()
   TryNewInstance<T>(array);
 
   TryAsMultiplexer<T>(array);
+
+  TryExtractArray<T>(array);
 }
 
 struct TryBasicVTKmType
@@ -388,6 +410,10 @@ struct TryBasicVTKmType
   {
     vtkm::cont::UnknownArrayHandle array = CreateArrayUnknown(T());
 
+    VTKM_TEST_ASSERT(array.GetValueTypeName() == vtkm::cont::TypeToString<T>());
+    VTKM_TEST_ASSERT(array.GetStorageTypeName() ==
+                     vtkm::cont::TypeToString<vtkm::cont::StorageTagBasic>());
+
     CheckUnknownArray<vtkm::TypeListAll, VTKM_DEFAULT_STORAGE_LIST>(
       array, vtkm::VecTraits<T>::NUM_COMPONENTS);
 
@@ -405,7 +431,7 @@ void TryUnusualType()
     CheckUnknownArray<VTKM_DEFAULT_TYPE_LIST, VTKM_DEFAULT_STORAGE_LIST>(array, 1);
     VTKM_TEST_FAIL("CastAndCall failed to error for unrecognized type.");
   }
-  catch (vtkm::cont::ErrorBadValue&)
+  catch (vtkm::cont::ErrorBadType&)
   {
     std::cout << "  Caught exception for unrecognized type." << std::endl;
   }
@@ -453,6 +479,18 @@ struct CheckExtractedArray
       auto extractedData = extractedPortal.Get(valueIndex);
       VTKM_TEST_ASSERT(test_equal(originalData, extractedData));
     }
+
+    // Make sure an extracted array stuffed back into an UnknownArrayHandle works.
+    // This can happen when working with an extracted array that is passed to functions
+    // that are implemented with UnknownArrayHandle.
+    vtkm::cont::UnknownArrayHandle unknownArray{ extractedArray };
+
+    using ComponentType =
+      typename vtkm::VecTraits<typename ExtractedArray::ValueType>::BaseComponentType;
+    vtkm::cont::UnknownArrayHandle newBasic = unknownArray.NewInstanceBasic();
+    newBasic.AsArrayHandle<vtkm::cont::ArrayHandleRuntimeVec<ComponentType>>();
+    vtkm::cont::UnknownArrayHandle newFloat = unknownArray.NewInstanceFloatBasic();
+    newFloat.AsArrayHandle<vtkm::cont::ArrayHandleRuntimeVec<vtkm::FloatDefault>>();
   }
 };
 
@@ -518,6 +556,87 @@ void TrySetMultiplexerArray()
   CheckUnknownArray<vtkm::List<vtkm::Id>, vtkm::List<VTKM_DEFAULT_STORAGE_TAG>>(unknownArray, 1);
 }
 
+template <typename T, typename BasicComponentType = typename vtkm::VecFlat<T>::ComponentType>
+void TryConvertRuntimeVec()
+{
+  using BasicArrayType = vtkm::cont::ArrayHandle<T>;
+  constexpr vtkm::IdComponent numFlatComponents = vtkm::VecFlat<T>::NUM_COMPONENTS;
+  using RuntimeArrayType = vtkm::cont::ArrayHandleRuntimeVec<BasicComponentType>;
+
+  std::cout << "    Get basic array as ArrayHandleRuntimeVec" << std::endl;
+  BasicArrayType inputArray;
+  inputArray.Allocate(ARRAY_SIZE);
+  SetPortal(inputArray.WritePortal());
+
+  vtkm::cont::UnknownArrayHandle unknownWithBasic{ inputArray };
+  VTKM_TEST_ASSERT(unknownWithBasic.GetNumberOfComponentsFlat() == numFlatComponents);
+
+  VTKM_TEST_ASSERT(unknownWithBasic.CanConvert<RuntimeArrayType>());
+  RuntimeArrayType runtimeArray = unknownWithBasic.AsArrayHandle<RuntimeArrayType>();
+
+  // Hack to convert the array handle to a flat array to make it easy to check the runtime array
+  vtkm::cont::ArrayHandle<vtkm::VecFlat<T>> flatInput{ inputArray.GetBuffers() };
+  VTKM_TEST_ASSERT(test_equal_ArrayHandles(flatInput, runtimeArray));
+
+  std::cout << "    Get ArrayHandleRuntimeVec as basic array" << std::endl;
+  vtkm::cont::UnknownArrayHandle unknownWithRuntimeVec{ runtimeArray };
+  VTKM_TEST_ASSERT(unknownWithRuntimeVec.GetNumberOfComponentsFlat() == numFlatComponents);
+
+  VTKM_TEST_ASSERT(unknownWithRuntimeVec.CanConvert<RuntimeArrayType>());
+  VTKM_TEST_ASSERT(unknownWithRuntimeVec.CanConvert<BasicArrayType>());
+  BasicArrayType outputArray = unknownWithRuntimeVec.AsArrayHandle<BasicArrayType>();
+  VTKM_TEST_ASSERT(test_equal_ArrayHandles(inputArray, outputArray));
+
+  std::cout << "    Copy ArrayHandleRuntimeVec to a new instance" << std::endl;
+  vtkm::cont::UnknownArrayHandle unknownCopy = unknownWithRuntimeVec.NewInstance();
+  VTKM_TEST_ASSERT(unknownWithRuntimeVec.GetNumberOfComponentsFlat() ==
+                   unknownCopy.GetNumberOfComponentsFlat());
+  vtkm::cont::ArrayCopy(unknownWithRuntimeVec, unknownCopy);
+  VTKM_TEST_ASSERT(test_equal_ArrayHandles(inputArray, unknownCopy));
+
+  std::cout << "    Copy ArrayHandleRuntimeVec as basic array" << std::endl;
+  unknownCopy = unknownWithRuntimeVec.NewInstanceBasic();
+  VTKM_TEST_ASSERT(unknownWithRuntimeVec.GetNumberOfComponentsFlat() ==
+                   unknownCopy.GetNumberOfComponentsFlat());
+  vtkm::cont::ArrayCopy(unknownWithRuntimeVec, unknownCopy);
+  VTKM_TEST_ASSERT(test_equal_ArrayHandles(inputArray, unknownCopy));
+
+  std::cout << "    Copy ArrayHandleRuntimeVec to float array" << std::endl;
+  unknownCopy = unknownWithRuntimeVec.NewInstanceFloatBasic();
+  VTKM_TEST_ASSERT(unknownWithRuntimeVec.GetNumberOfComponentsFlat() ==
+                   unknownCopy.GetNumberOfComponentsFlat());
+  vtkm::cont::ArrayCopy(unknownWithRuntimeVec, unknownCopy);
+  VTKM_TEST_ASSERT(test_equal_ArrayHandles(inputArray, unknownCopy));
+}
+
+void TryConvertRuntimeVec()
+{
+  std::cout << "  Scalar array." << std::endl;
+  TryConvertRuntimeVec<vtkm::FloatDefault>();
+
+  std::cout << "  Equivalent scalar." << std::endl;
+  TryConvertRuntimeVec<VTKM_UNUSED_INT_TYPE>();
+
+  std::cout << "  Basic Vec." << std::endl;
+  TryConvertRuntimeVec<vtkm::Id3>();
+
+  std::cout << "  Vec of Vecs." << std::endl;
+  TryConvertRuntimeVec<vtkm::Vec<vtkm::Vec2f, 3>>();
+
+  std::cout << "  Vec of Vecs of Vecs." << std::endl;
+  TryConvertRuntimeVec<vtkm::Vec<vtkm::Vec<vtkm::Id4, 3>, 2>>();
+
+  std::cout << "  Compatible but different C types." << std::endl;
+  if (sizeof(int) == sizeof(long))
+  {
+    TryConvertRuntimeVec<vtkm::Vec<int, 4>, long>();
+  }
+  else // assuming sizeof(long long) == sizeof(long)
+  {
+    TryConvertRuntimeVec<vtkm::Vec<long long, 4>, long>();
+  }
+}
+
 struct DefaultTypeFunctor
 {
   template <typename T>
@@ -549,6 +668,9 @@ void TestUnknownArrayHandle()
 
   std::cout << "Try setting ArrayHandleMultiplexer" << std::endl;
   TrySetMultiplexerArray();
+
+  std::cout << "Try converting between ArrayHandleRuntimeVec and basic array" << std::endl;
+  TryConvertRuntimeVec();
 }
 
 } // anonymous namespace
